import std/json, std/options, std/strutils, std/times
import uuids

type
  MetaSource* = enum msTagInfo = "tag info", msFileLocation = "file location"

  BaseModel* = object of RootObj
    id*: UUID
    name*: string

  Album* = object of BaseModel
    imageUri*: Option[string]
    trackTotal: int
    year*: Option[int]

  Artist* = object of BaseModel
    imageUri*: Option[string]

  Bookmark* = object of BaseModel
    playlistId*: UUID
    mediaFileId*: UUID
    playIdx*: int
    playTimeMs*: int
    userCreated*: bool
    createdAt*: DateTime
    lastUsed*: DateTime

  MediaFile* = object of BaseModel
    comment*: string
    dateAdded*: DateTime
    discNumber*: Option[string]
    fileHash*: string
    filePath*: string
    imageUri*: Option[string]
    lastPlayed*: DateTime
    metaInfoSource*: MetaSource
    playCount*: int
    presentLocally*: bool
    trackNumber*: Option[int]

  Playlist* = object of BaseModel
    userCreated*: bool
    mediaFileCount*: int
    copiedFromId: Option[UUID]
    createdAt*: DateTime
    lastUsed*: DateTime

  Tag* = object
    name*: string
    description*: Option[string]

proc `$`*(mf: MediaFile): string =
  if mf.trackNumber.isSome: align($mf.trackNumber.get, 2) & " - " & mf.name
  else: mf.name

proc `$`*(a: Album): string =
  result = a.name
  if a.year.isSome: result &= " (" & $a.year & ")"

proc `$`*(m: BaseModel): string = m.name

proc `$`*(t: Tag): string =
  result = t.name
  if t.description.isSome: result &= t.description.get
